import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useState, useEffect, Children, useCallback } from 'react';
import { Datagrid } from 'react-admin';
import isEmpty from 'lodash/isEmpty';
import filter from 'lodash/filter';
import get from 'lodash/get';
import { ViewColumn as ColumnIcon } from '@mui/icons-material';
import Button from '@mui/material/Button';
import SelectionDialog from './SelectionDialog';
import { setValueInLocalStorage, getValueFromLocalStorage } from './utils';
const arrayToSelection = (values) => values.reduce((selection, columnName) => {
    selection[columnName] = true;
    return selection;
}, {});
const CustomizableDatagrid = ({ children, defaultColumns, resource, ...rest }) => {
    const [modalOpened, setModalOpened] = useState(false);
    const [selection, setSelection] = useState({});
    const toggleColumn = useCallback((columnName) => {
        setSelection((prevState) => ({
            ...prevState,
            [columnName]: !prevState[columnName] || false,
        }));
    }, [selection]);
    const getColumnLabels = useCallback(() => filter(React.Children.map(children, (field) => field && {
        source: get(field, ['props', 'source']),
        label: get(field, ['props', 'label']),
    }), (item) => item && item.source), [children]);
    const renderChild = useCallback((child) => {
        const source = get(child, ['props', 'source']);
        if (!source || !selection[source]) {
            return React.cloneElement(child, {});
        }
        return null;
    }, []);
    useEffect(() => {
        setValueInLocalStorage(resource, selection);
    }, [selection]);
    useEffect(() => {
        const columnNames = Children.map(children, (field) => get(field, ['props', 'source']));
        const previousSelection = getValueFromLocalStorage(resource);
        if (!isEmpty(previousSelection)) {
            setSelection(previousSelection);
            return;
        }
        if (!isEmpty(defaultColumns)) {
            setSelection(arrayToSelection(defaultColumns));
            return;
        }
        setSelection(arrayToSelection(columnNames || []));
    }, []);
    return (_jsxs("div", { children: [_jsx("div", { style: { float: 'right', marginRight: '1rem' }, children: _jsx(Button, { variant: "outlined", size: "small", "aria-label": "add", onClick: () => setModalOpened(true), children: _jsx(ColumnIcon, {}) }) }), modalOpened && (_jsx(SelectionDialog, { selection: selection, columns: getColumnLabels(), onColumnClicked: toggleColumn, onClose: () => setModalOpened(false), resource: resource })), _jsx(Datagrid, { ...rest, children: React.Children.map(children, renderChild) })] }));
};
export default CustomizableDatagrid;
